from typing import List
import subete

TEST_PATH: str = "tests/python/"
TEST_LANG: str = "python"
TEST_FILES: List[str] = ["hello_world.py", "testinfo.yml", "README.md"]
TEST_PROJECTS: List[subete.Project] = [
    subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False}), 
    subete.Project("reverse-string", {"words": ["reverse", "string"], "requires_parameters": True}),
]

UNTESTABLE_PATH: str = "tests/mathematica"
UNTESTABLE_LANG: str = "mathematica"
UNTESTABLE_FILES: List[str] = ["reverse-string.nb", "untestable.yml"]


def test_language_collection_str():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert str(test) == "Python"


def test_language_collection_name():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.name() == "Python"


def test_language_collection_test_file():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.testinfo() is not None


def test_language_collection_untestable_file():
    test = subete.LanguageCollection(UNTESTABLE_LANG, UNTESTABLE_PATH, UNTESTABLE_FILES, TEST_PROJECTS)
    assert test.untestable_info() is not None


def test_language_collection_readme():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.readme() is not None


def test_language_collection_sample_programs():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test is not None
    assert subete.SampleProgram(TEST_PATH, TEST_FILES[0], test) in test


def test_language_collection_total_programs():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.total_programs() == 1


def test_language_collection_total_size():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.total_size() > 0


def test_language_collection_total_line_count():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.total_line_count() == 1


def test_language_collection_language_url():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.lang_docs_url() == "https://sampleprograms.io/languages/python"


def test_missing_programs():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.missing_programs() == [TEST_PROJECTS[1]]


def test_missing_programs_count():
    test = subete.LanguageCollection(TEST_LANG, TEST_PATH, TEST_FILES, TEST_PROJECTS)
    assert test.missing_programs_count() == 1



from typing import List
import subete
from subete.repo import LanguageCollection

TEST_PATH: str = "tests/python/"
TEST_LANG: str = "python"
TEST_FILES: List[str] = ["hello_world.py", "testinfo.yml", "README.md"]
TEST_PROJECTS: List[subete.Project] = [
    subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False}), 
    subete.Project("reverse-string", {"words": ["reverse", "string"], "requires_parameters": True}),
    subete.Project("rot13", {"words": ["rot13"], "requires_parameters": False}),
]
TEST_LANG_COLLECTION: subete.LanguageCollection = subete.LanguageCollection(
    TEST_LANG, 
    TEST_PATH, 
    TEST_FILES, 
    TEST_PROJECTS
)


def test_sample_program_str():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert str(test) == "Hello World in Python"


def test_sample_program_language():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.language_collection() == TEST_LANG_COLLECTION


def test_sample_program_code():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.code() == 'print("Hello, World!")\n'


def test_sample_program_line_count():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.line_count() == 1


def test_sample_program_size():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.size() > 0


def test_sample_program_requirements_url():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.project().requirements_url(
    ) == "https://sampleprograms.io/projects/hello-world"


def test_sample_program_documentation_url():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.documentation_url() == "https://sampleprograms.io/projects/hello-world/python"


def test_sample_program_issue_query_url():
    test = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test.article_issue_query_url(
    ) == "https://github.com//TheRenegadeCoder/sample-programs-website/issues?utf8=%E2%9C%93&q=is%3Aissue+is%3Aopen+hello+world+python"


def test_generate_project_hyphen_branch():
    test = subete.SampleProgram(
        "tests/c", 
        "rot13.c", 
        LanguageCollection(
            "c",
            "tests/c",
            ["rot13.c", "testinfo.yml", "README.md"],
            TEST_PROJECTS
        )
    )
    assert test.project_pathlike_name() == "rot13"


def test_sample_program_equality():
    test1 = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    test2 = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    assert test1 == test2


def test_sample_program_inequality():
    test1 = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    test2 = subete.SampleProgram(
        "tests/c", 
        "rot13.c", 
        LanguageCollection(
            "c",
            "tests/c",
            ["rot13.c", "testinfo.yml", "README.md"],
            TEST_PROJECTS
        )
    )
    assert test1 != test2


def test_sample_program_diff_class():
    test1 = subete.SampleProgram(TEST_PATH, TEST_FILES[0], TEST_LANG_COLLECTION)
    test2 = "foo"
    assert test1 != test2



import tempfile
from pathlib import Path

import git
import pytest

import subete


def test_bad_repo_total_projects(bad_test_repo):
    assert bad_test_repo.total_approved_projects() == 1


def test_bad_repo_languages(bad_test_repo):
    assert len(list(bad_test_repo)) == 1


def test_bad_repo_total_programs(bad_test_repo):
    assert bad_test_repo.total_programs() == 0


def test_bad_repo_total_tests(bad_test_repo):
    assert bad_test_repo.total_tests() == 0


@pytest.fixture(scope="module")
def bad_test_repo():
    with tempfile.TemporaryDirectory() as repo_dir, tempfile.TemporaryDirectory() as website_repo_dir:
        # Create repo
        repo: git.Repo = git.Repo.init(repo_dir)

        archive_dir = f"{repo_dir}/archive/f/foo"
        sample_program_file = f"{archive_dir}/whatever.foo"
        Path(archive_dir).mkdir(parents=True)
        Path(sample_program_file).write_text("hello\n")

        repo.index.add([sample_program_file])
        repo.index.commit("Initial commit")

        # Create website repo
        website_repo: git.Repo = git.Repo.init(website_repo_dir)

        project_doc_dir = f"{website_repo_dir}/sources/projects/bad"
        project_doc_file = f"{project_doc_dir}/something.md"
        Path(project_doc_dir).mkdir(parents=True)
        Path(project_doc_file).write_text("hello\n")

        website_repo.index.add([project_doc_file])
        website_repo.index.commit("Initial commit")

        yield subete.load(repo_dir, website_repo_dir)

        repo.close()
        website_repo.close()



from unittest.mock import patch
import tempfile

import pytest

import subete

SAMPLE_PROGRAMS_TEMP_DIR = tempfile.TemporaryDirectory()
SAMPLE_PROGRAMS_WEBSITE_TEMP_DIR = tempfile.TemporaryDirectory()


def test_doc_url_multiword_lang(test_repo):
    language: subete.LanguageCollection = test_repo["Commodore Basic"]
    assert language.lang_docs_url() == "https://sampleprograms.io/languages/commodore-basic"


def test_doc_url_symbol_lang(test_repo):
    language: subete.LanguageCollection = test_repo["C#"]
    assert language.lang_docs_url() == "https://sampleprograms.io/languages/c-sharp"


def test_testinfo_url_multiword_lang(test_repo):
    language: subete.LanguageCollection = test_repo["Commodore Basic"]
    assert language.testinfo_url(
    ) == "https://github.com/TheRenegadeCoder/sample-programs/blob/main/archive/c/commodore-basic/testinfo.yml"


def test_testinfo_url_symbol_lang(test_repo):
    language: subete.LanguageCollection = test_repo["C#"]
    assert language.testinfo_url(
    ) == "https://github.com/TheRenegadeCoder/sample-programs/blob/main/archive/c/c-sharp/testinfo.yml"


def test_untesting_info_url(test_repo):
    language: subete.LanguageCollection = test_repo["Mathematica"]
    assert language.untestable_info_url(
    ) == "https://github.com/TheRenegadeCoder/sample-programs/blob/main/archive/m/mathematica/untestable.yml"


def test_requirements_url_multiword_lang(test_repo):
    program: subete.SampleProgram = test_repo["Commodore Basic"]["Hello World"]
    assert program.project().requirements_url(
    ) == "https://sampleprograms.io/projects/hello-world"


def test_requirements_url_symbol_lang(test_repo):
    program: subete.SampleProgram = test_repo["C#"]["Hello World"]
    assert program.project().requirements_url(
    ) == "https://sampleprograms.io/projects/hello-world"


def test_documentation_url_multiword_lang(test_repo):
    program: subete.SampleProgram = test_repo["Commodore Basic"]["Hello World"]
    assert program.documentation_url(
    ) == "https://sampleprograms.io/projects/hello-world/commodore-basic"


def test_documentation_url_symbol_lang(test_repo):
    program: subete.SampleProgram = test_repo["C#"]["Hello World"]
    assert program.documentation_url(
    ) == "https://sampleprograms.io/projects/hello-world/c-sharp"


def test_article_issue_url_multiword_lang(test_repo):
    program: subete.SampleProgram = test_repo["Commodore Basic"]["Hello World"]
    assert program.article_issue_query_url(
    ) == "https://github.com//TheRenegadeCoder/sample-programs-website/issues?utf8=%E2%9C%93&q=is%3Aissue+is%3Aopen+hello+world+commodore+basic"


def test_article_issue_url_symbol_lang(test_repo):
    program: subete.SampleProgram = test_repo["C#"]["Hello World"]
    assert program.article_issue_query_url(
    ) == "https://github.com//TheRenegadeCoder/sample-programs-website/issues?utf8=%E2%9C%93&q=is%3Aissue+is%3Aopen+hello+world+c#"


def test_authors(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert "Jeremy Griffith" in program.authors()


def test_created_not_none(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert program.created() is not None


def test_modified_not_none(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert program.modified() is not None


def test_code(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert program.code() == "print('Hello, World!')\n"


@pytest.mark.parametrize(
    "language,expected_result",
    [
        ("Piet", "png"),
        ("Python", ""),
    ]
)
def test_image_type(language, expected_result, test_repo):
    program: subete.SampleProgram = test_repo[language]["Hello World"]
    assert program.image_type() == expected_result


def test_doc_authors(test_repo):
    program: subete.SampleProgram = test_repo["Yoix"]["Hello World"]
    assert "rzuckerm" in program.doc_authors()


def test_doc_created(test_repo):
    program: subete.SampleProgram = test_repo["Yoix"]["Hello World"]
    assert program.doc_created() is not None


def test_doc_modified(test_repo):
    program: subete.SampleProgram = test_repo["Yoix"]["Hello World"]
    assert program.doc_modified() is not None


def test_project_path(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert program.project_path().endswith("/archive/p/python/hello_world.py")


def test_project_has_test(test_repo):
    program: subete.SampleProgram = test_repo["Commodore Basic"]["Hello World"]
    assert program.project().has_testing()


def test_repo_languages(test_repo):
    assert len(list(test_repo)) > 0


def test_repo_total_programs(test_repo):
    assert test_repo.total_programs() > 0


def test_repo_total_tests(test_repo):
    assert test_repo.total_tests() > 0


def test_repo_total_untestables(test_repo):
    assert test_repo.total_untestables() > 0


def test_repo_languages_by_letter(test_repo):
    assert len(test_repo.languages_by_letter("p")) > 0


def test_random_program(test_repo):
    assert test_repo.random_program() != test_repo.random_program()


def test_approved_projects(test_repo):
    assert len(test_repo.approved_projects()) > 0


def test_total_approved_projects(test_repo):
    assert test_repo.total_approved_projects() > 0


def test_sorted_language_letters(test_repo):
    assert "p" in test_repo.sorted_language_letters()


def test_program_has_docs(test_repo):
    program: subete.SampleProgram = test_repo["Python"]["Hello World"]
    assert program.has_docs()


def test_sample_programs_repo_dir(test_repo):
    assert test_repo.sample_programs_repo_dir() == SAMPLE_PROGRAMS_TEMP_DIR.name


def test_language_has_docs(test_repo):
    language: subete.LanguageCollection = test_repo["Befunge"]
    assert language.has_docs()


def test_language_doc_author(test_repo):
    language: subete.LanguageCollection = test_repo["Befunge"]
    assert "Stuart Irwin" in language.doc_authors()


def test_language_doc_created(test_repo):
    language: subete.LanguageCollection = test_repo["Befunge"]
    assert language.doc_created() is not None


def test_language_doc_modified(test_repo):
    language: subete.LanguageCollection = test_repo["Befunge"]
    assert language.doc_modified() is not None


def test_project_doc_author(test_repo):
    project: subete.Project = test_repo["Python"]["Selection Sort"].project()
    assert "Parker Johansen" in project.doc_authors()


def test_project_doc_created(test_repo):
    project: subete.Project = test_repo["Python"]["Selection Sort"].project()
    assert project.doc_created() is not None


def test_project_doc_modified(test_repo):
    project: subete.Project = test_repo["Python"]["Selection Sort"].project()
    assert project.doc_modified() is not None


@pytest.fixture(scope="module")
def test_repo():
    with patch("subete.repo.tempfile.TemporaryDirectory") as mock:
        mock.side_effect = [SAMPLE_PROGRAMS_TEMP_DIR, SAMPLE_PROGRAMS_WEBSITE_TEMP_DIR]
        yield subete.load()

    SAMPLE_PROGRAMS_TEMP_DIR.cleanup()
    SAMPLE_PROGRAMS_WEBSITE_TEMP_DIR.cleanup()



import subete


def test_project_str():
    test = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    assert str(test) == "Hello World"


def test_name():
    test = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    assert test.name() == "Hello World"


def test_pathlike_name():
    test = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    assert test.pathlike_name() == "hello-world"


def test_has_testing():
    test = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    assert test.has_testing()


def test_project_equality():
    test1 = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    test2 = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    assert test1 == test2


def test_project_inequality():
    test1 = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    test2 = subete.Project("goodbye-world", {"words": ["goodbye", "world"], "requires_parameters": False})
    assert test1 != test2


def test_project_diff_class():
    test1 = subete.Project("hello-world", {"words": ["hello", "world"], "requires_parameters": False})
    test2 = "hello"
    assert test1 != test2
